// Copyright 2021 Red Hat, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use crate::NisporError;

pub(crate) fn parse_as_mac(
    mac_len: usize,
    data: &[u8],
) -> Result<String, NisporError> {
    let mut rt = String::new();
    for i in 0..mac_len {
        rt.push_str(&format!(
            "{:02x}",
            *data
                .get(i)
                .ok_or(NisporError::bug("wrong index at mac parsing".into()))?
        ));
        if i != mac_len - 1 {
            rt.push_str(":");
        }
    }
    Ok(rt)
}
