// Copyright 2021 Red Hat, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use thiserror::Error;

use netlink_packet_core::{ErrorMessage, NetlinkMessage};

use crate::EthtoolMessage;

#[derive(Clone, Eq, PartialEq, Debug, Error)]
pub enum EthtoolError {
    #[error("Received an unexpected message {0:?}")]
    UnexpectedMessage(NetlinkMessage<EthtoolMessage>),

    #[error("Received a netlink error message {0}")]
    NetlinkError(ErrorMessage),

    #[error("A netlink request failed")]
    RequestFailed(String),
}
