// Copyright 2021 Red Hat, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

use crate::{EthtoolHandle, PauseGetRequest};

pub struct PauseHandle(EthtoolHandle);

impl PauseHandle {
    pub fn new(handle: EthtoolHandle) -> Self {
        PauseHandle(handle)
    }

    /// Retrieve the pause setting of a interface (equivalent to `ethtool -a eth1`)
    pub fn get(&mut self, iface_name: Option<&str>) -> PauseGetRequest {
        PauseGetRequest::new(self.0.clone(), iface_name)
    }
}
